BEGIN;

 --   CREATE EXTENSION IF NOT EXISTS "uuid-ossp";

    DO $$
    BEGIN
        IF NOT EXISTS (SELECT 1 FROM pg_type WHERE typname = 'system') THEN
            CREATE TYPE system AS enum('SFDC', 'LOCAL');
        END IF;

        IF NOT EXISTS (SELECT 1 FROM pg_type WHERE typname = 'status') THEN
            CREATE TYPE status AS enum('ACTIVE', 'INACTIVE');
        END IF;

        IF NOT EXISTS (SELECT 1 FROM pg_type WHERE typname = 'origin') THEN
            CREATE TYPE origin AS enum('SFDC', 'DELL', 'CPN', 'LOCAL');
        END IF;
    END
    $$;

    CREATE TABLE IF NOT EXISTS entitlements (
        created_at timestamp DEFAULT now(),
        updated_at timestamp DEFAULT now(),
        deleted_at timestamp,

        is_deleted boolean DEFAULT false,
        entitlement_id uuid PRIMARY KEY DEFAULT uuid_generate_v4(),
        org_key text NOT NULL,
        external_id text NOT NULL,

        /* entitlement should have only one product */
        product_id text NOT NULL,

        origin origin NOT NULL,
        system system NOT NULL,
        status status DEFAULT 'ACTIVE',
        begin_date timestamp,
        end_date timestamp,
        licenses int NOT NULL,
        eval boolean DEFAULT TRUE,
        config jsonb,
        custom_attr jsonb,

        CONSTRAINT entitlements_product_id_system_fkey
            FOREIGN KEY (product_id, system)
            REFERENCES products(product_id, system) ON UPDATE CASCADE
    );

    CREATE INDEX IF NOT EXISTS entitlements_org_key_idx ON entitlements(org_key);
    CREATE INDEX IF NOT EXISTS entitlements_status_idx ON entitlements(status);
    CREATE INDEX IF NOT EXISTS entitlements_begin_date_idx ON  entitlements((begin_date::DATE));
    CREATE INDEX IF NOT EXISTS entitlements_end_date_idx ON  entitlements((end_date::DATE));

COMMIT;
