BEGIN;
    -- Soft DELETE already duplicated entitlements by preserving the last inserted one
    WITH T AS (
        SELECT *,
        row_number() over (partition by org_key, external_id, origin, product_id, system order by created_at desc) rn
        FROM entitlements
        WHERE is_deleted = false
    )
    UPDATE entitlements
    SET
        is_deleted = true,
        deleted_at = 'now'::timestamp,
        deleted_by = 'MIGRATION'
    WHERE
        entitlement_id IN (
            SELECT
                T.entitlement_id
            FROM T
            WHERE T.rn > 1
        );

    CREATE UNIQUE INDEX IF NOT EXISTS
        entitlements_org_key_external_id_origin_product_id_system_idx
        ON entitlements (
            org_key,
            external_id,
            origin,
            product_id,
            system
        )
    WHERE is_deleted = false;
COMMIT;
