BEGIN;

    DO $$
        BEGIN
            IF NOT EXISTS (SELECT 1 FROM pg_type WHERE typname = 'compliance_status') THEN
                CREATE TYPE compliance_status AS enum('PENDING', 'PROGRESS','SUCCESS', 'ERROR', 'CANCEL');
            END IF;
        END
    $$;

    CREATE TABLE IF NOT EXISTS compliance (
        created_at timestamp DEFAULT now(),
        created_by text,
        updated_at timestamp DEFAULT now(),
        updated_by text,
        deleted_at timestamp,
        deleted_by text,
        is_deleted boolean DEFAULT false,

        compliance_id uuid PRIMARY KEY DEFAULT uuid_generate_v4(),
        id serial NOT NULL,
        sguid text NOT NULL,
        name text NOT NULL,
        org_key text NOT NULL,
        status compliance_status DEFAULT 'PENDING',
        holding_period_end timestamp NOT NULL,

        service jsonb NOT NULL,
        messages jsonb NOT NULL
    );


    CREATE UNIQUE INDEX IF NOT EXISTS
        compliance_sguid_org_key_idx
        ON compliance (
            sguid,
            org_key
        )
    WHERE is_deleted = false;

COMMIT;
