BEGIN;

DO $$
    BEGIN
        IF NOT EXISTS (SELECT 1 FROM pg_type WHERE typname = 'deregistration_status') THEN
            CREATE TYPE deregistration_status AS enum('WAIT', 'HOLD','PURGE', 'CANCEL');
        END IF;
    END
$$;

CREATE TABLE IF NOT EXISTS deregistrations (
    created_at timestamp DEFAULT now(),
    created_by text,
    updated_at timestamp DEFAULT now(),
    updated_by text,
    deleted_at timestamp,
    deleted_by text,
    is_deleted boolean DEFAULT false,

    id serial NOT NULL,
    deregistration_id uuid PRIMARY KEY DEFAULT uuid_generate_v4(),
    org_key text NOT NULL,
    execution_id uuid NOT NULL,
    status deregistration_status DEFAULT 'WAIT',
    wait_begin_datetime timestamp NULL,
    hold_begin_datetime timestamp NULL,
    purge_begin_datetime timestamp NULL,
    CONSTRAINT deregistrations_execution_id_fkey
      FOREIGN KEY (execution_id)
          REFERENCES executions(execution_id) ON UPDATE CASCADE
);

CREATE INDEX IF NOT EXISTS deregistrations_wait_begin_datetime_idx ON deregistrations(status, wait_begin_datetime)
    WHERE status = 'WAIT' AND wait_begin_datetime IS NOT NULL AND is_deleted = false;
CREATE INDEX IF NOT EXISTS deregistrations_hold_begin_datetime_idx ON deregistrations(status, hold_begin_datetime)
    WHERE status = 'HOLD' AND hold_begin_datetime IS NOT NULL AND is_deleted = false;
CREATE INDEX IF NOT EXISTS deregistrations_purge_begin_datetime_idx ON deregistrations(status, purge_begin_datetime)
    WHERE status = 'PURGE' AND purge_begin_datetime IS NOT NULL AND is_deleted = false;
CREATE UNIQUE INDEX IF NOT EXISTS deregistrations_org_key_idx ON deregistrations(org_key)
    WHERE is_deleted = false;

COMMIT;
