BEGIN;

ALTER TABLE executions
    ADD COLUMN IF NOT EXISTS details_tsvector tsvector;
UPDATE executions
SET details_tsvector = jsonb_to_tsvector('english', details, '["string", "numeric", "boolean"]')
WHERE is_deleted = false;
ALTER TABLE executions
    ALTER COLUMN details_tsvector SET NOT NULL;

CREATE OR REPLACE FUNCTION update_executions_details_tsvector()
    RETURNS TRIGGER AS
$$
BEGIN
    NEW.details_tsvector := jsonb_to_tsvector('english', NEW.details, '["string", "numeric", "boolean"]');
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

DROP TRIGGER IF EXISTS update_executions_details_tsvector_trigger ON executions;
CREATE TRIGGER update_executions_details_tsvector_trigger
    BEFORE INSERT OR UPDATE OF details
    ON executions
    FOR EACH ROW
EXECUTE PROCEDURE update_executions_details_tsvector();

-- CONCURRENTLY can't be used here, cause we are in a transaction, so the executions table will be locked for writes till the index creation is ready
CREATE INDEX IF NOT EXISTS executions_details_tsvector_idx ON executions USING gin (details_tsvector) WHERE is_deleted = false;
CREATE INDEX IF NOT EXISTS executions_created_at_idx ON executions (created_at DESC);
-- This index was added, but not used
DROP INDEX IF EXISTS executions_details_full_text_search_idx;

COMMIT;
