BEGIN;

  ALTER TABLE compliance
      ADD COLUMN IF NOT EXISTS details_tsvector tsvector;

  UPDATE compliance
    SET details_tsvector = jsonb_to_tsvector('english', jsonb_build_object(
          'created_at', created_at,
          'created_by', created_by,
          'updated_at', updated_at,
          'updated_by', updated_by,

          'compliance_id', compliance_id,
          'sguid', sguid,
          'name', name,
          'org_key', org_key,
          'status', status,

          'service', service,
          'messages', messages
    ), '["string", "numeric", "boolean"]')
      WHERE is_deleted = false;

  ALTER TABLE compliance
      ALTER COLUMN details_tsvector SET NOT NULL;

  CREATE OR REPLACE FUNCTION update_compliance_details_tsvector()
      RETURNS TRIGGER AS
  $$
  BEGIN
      NEW.details_tsvector := jsonb_to_tsvector('english', jsonb_build_object(
          'created_at', NEW.created_at,
          'created_by', NEW.created_by,
          'updated_at', NEW.updated_at,
          'updated_by', NEW.updated_by,

          'compliance_id', NEW.compliance_id,
          'sguid', NEW.sguid,
          'name', NEW.name,
          'org_key', NEW.org_key,
          'status', NEW.status,

          'service', NEW.service,
          'messages', NEW.messages
      ), '["string", "numeric", "boolean"]');
      RETURN NEW;
  END;
  $$ LANGUAGE plpgsql;

  DROP TRIGGER IF EXISTS update_compliance_details_tsvector_trigger ON compliance;
  CREATE TRIGGER update_compliance_details_tsvector_trigger
      BEFORE INSERT OR UPDATE
      ON compliance
      FOR EACH ROW
        EXECUTE PROCEDURE update_compliance_details_tsvector();

  CREATE INDEX IF NOT EXISTS compliance_created_at_idx ON compliance (created_at DESC);

COMMIT;
