BEGIN;

    DO $$
    BEGIN
        IF NOT EXISTS (SELECT 1 FROM pg_type WHERE typname = 'product_type') THEN
            CREATE TYPE product_type AS enum('DEFAULT', 'SINGLETON');
        END IF;
    END
    $$;

    ALTER TABLE products ADD COLUMN IF NOT EXISTS type product_type NOT NULL DEFAULT 'DEFAULT';

    INSERT INTO features (
        feature_id,
        internal_name,
        external_name,
        ui_name,
        ui_abbr,
        requires,
        conflicts
    ) VALUES (
        'psc:feature:graceperiod',
        'CBC Grace Period',
        'VMware Carbon Black Grace Period',
        'Grace Period',
        'GP',
        '{ "cmp_type": "ANY", "features": [] }'::jsonb,
        '{ "cmp_type": "ANY", "features": [] }'::jsonb
    ) ON CONFLICT DO NOTHING;

    INSERT INTO feature_custom_attrs (
      feature_id,
      attr,
      descr,
      options
    ) VALUES (
      'psc:feature:graceperiod',
      'grace_period_days',
      'The interval of the grace period.',
      '[null, 0, 5, 10, 15, 30, 60, 90, 180]'
    ) ON CONFLICT DO NOTHING;

   INSERT INTO products (
        type,
        product_id,
        system,
        name,
        ui_name,
        ui_abbr
    ) VALUES (
        'SINGLETON',
        'graceperiod',
        'SFDC',
        'Grace Period',
        'Grace Period',
        'GP'
    ) ON CONFLICT DO NOTHING;

    INSERT INTO products (
        type,
        product_id,
        system,
        name,
        ui_name,
        ui_abbr
    ) VALUES (
        'SINGLETON',
        'graceperiod',
        'LOCAL',
        'Grace Period',
        'Grace Period',
        'GP'
    ) ON CONFLICT DO NOTHING;

    INSERT INTO product_features (
        product_id,
        feature_id,
        system
    ) VALUES (
        'graceperiod',
        'psc:feature:graceperiod',
        'SFDC'
    ) ON CONFLICT DO NOTHING;

    INSERT INTO product_features (
        product_id,
        feature_id,
        system
    ) VALUES (
        'graceperiod',
        'psc:feature:graceperiod',
        'LOCAL'
    ) ON CONFLICT DO NOTHING;

    CREATE UNIQUE INDEX IF NOT EXISTS entitlements_org_key_graceperiod_unique_idx
      ON entitlements (org_key, product_id)
        WHERE product_id = 'graceperiod'
          AND is_deleted = false;

COMMIT;
