BEGIN;

DO $$
    BEGIN
        IF NOT EXISTS (SELECT 1 FROM pg_type WHERE typname = 'org_stage') THEN
            CREATE TYPE org_stage AS enum(
                'UNENTITLED',
                'ENTITLED',
                'GRACE',
                'DEREGISTER_HOLD',
                'DEREGISTER_WAIT',
                'DEREGISTER_PURGE'
            );
        END IF;
    END
$$;

CREATE TABLE IF NOT EXISTS org_stage_transitions (
    created_at timestamp DEFAULT now(),
    created_by text,
    updated_at timestamp DEFAULT now(),
    updated_by text,
    deleted_at timestamp,
    deleted_by text,
    is_deleted boolean DEFAULT false,

    id serial,
    org_stage_transition_id uuid PRIMARY KEY DEFAULT uuid_generate_v4(),
    org_key text NOT NULL,
    stage org_stage NOT NULL,

    execution_id uuid NOT NULL,
    CONSTRAINT org_stage_transitions_execution_id_fkey
      FOREIGN KEY (execution_id)
          REFERENCES executions(execution_id) ON UPDATE CASCADE
);

CREATE TABLE IF NOT EXISTS org_stage_transition_execution
(
    created_at timestamp DEFAULT now(),
    created_by text,
    updated_at timestamp DEFAULT now(),
    updated_by text,
    deleted_at timestamp,
    deleted_by text,
    is_deleted boolean DEFAULT false,
    id serial,
    org_stage_transition_execution_id uuid PRIMARY KEY DEFAULT uuid_generate_v4(),
    org_key text NOT NULL,
    execution_id uuid NOT NULL,
    stage org_stage NOT NULL,
    retry_count integer DEFAULT 0 NOT NULL,
    retried_at timestamp,
    CONSTRAINT org_stage_transition_execution_id_fkey
        FOREIGN KEY (execution_id)
            REFERENCES executions(execution_id) ON UPDATE CASCADE
);

CREATE UNIQUE INDEX IF NOT EXISTS org_stage_transition_execution_org_key_idx ON org_stage_transition_execution(org_key) WHERE is_deleted = false;

COMMIT;
