CREATE TABLE IF NOT EXISTS fast_tenants (
    org_key text PRIMARY KEY,
    tenant_name text NOT NULL,
    serial_numbers text,
    erp_customer_id text,
    last_action text,
    last_execution_id uuid REFERENCES executions (execution_id),
    modified_at timestamp DEFAULT now()
);

-- TODO The trigger might be overkill, consult with ChrisB

CREATE OR REPLACE FUNCTION check_serial_numbers()
RETURNS TRIGGER AS $$
DECLARE
    serial_number text;
BEGIN
    FOREACH serial_number IN ARRAY string_to_array(NEW.serial_numbers, ',') LOOP
        IF NOT EXISTS (SELECT 1 FROM entitlements WHERE external_id = serial_number) THEN
            RAISE EXCEPTION 'serial_number, which maps to external_id, % not found in entitlements table', serial_number;
END IF;
END LOOP;
RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER serial_number_trigger
    BEFORE INSERT OR UPDATE ON fast_tenants
        FOR EACH ROW EXECUTE FUNCTION check_serial_numbers();
